@@ -40,7 +40,7 @@ that the Quartznet model expects it.
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-# Copyright (c) 2021 scart97
+# Copyright (c) 2021-2022 scart97
 
 __all__ = [
     "FeatureBatchNormalizer",
@@ -52,14 +52,19 @@ __all__ = [
 ]
 
 import math
-from dataclasses import dataclass
-from typing import Optional
+from typing import Optional, Tuple
 
 import torch
 from torch import nn
-from torchaudio.functional import create_fb_matrix
+from torchaudio.functional import melscale_fbanks
 
-from thunder.blocks import convolution_stft
+from thunder.blocks import (
+    Masked,
+    MultiSequential,
+    convolution_stft,
+    lengths_to_mask,
+    normalize_tensor,
+)
 
 
 class FeatureBatchNormalizer(nn.Module):
@@ -68,21 +73,21 @@ class FeatureBatchNormalizer(nn.Module):
         super().__init__()
         self.div_guard = 1e-5
 
-    @torch.no_grad()
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
+    def forward(
+        self, x: torch.Tensor, lengths: torch.Tensor
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
         """
         Args:
-            x : Tensor of shape (batch, features, time)
+            x: Tensor of shape (batch, features, time)
         """
-        mask = x.abs() > 0.0
-        num_elements = mask.sum(dim=2, keepdim=True).detach()
-        x_mean = x.sum(dim=2, keepdim=True).detach() / num_elements
-        numerator = (x - x_mean).pow(2).sum(dim=2, keepdim=True).detach()
-        x_std = (numerator / num_elements).sqrt()
-        # make sure x_std is not zero
-        x_std += self.div_guard
-        result = (x - x_mean) / x_std
-        return torch.masked_fill(result, ~mask, 0.0)
+        # https://github.com/pytorch/pytorch/issues/45208
+        # https://github.com/pytorch/pytorch/issues/44768
+        with torch.no_grad():
+            mask = lengths_to_mask(lengths, x.shape[-1])
+            return (
+                normalize_tensor(x, mask.unsqueeze(1), div_guard=self.div_guard),
+                lengths,
+            )
 
 
 class DitherAudio(nn.Module):
@@ -94,7 +99,7 @@ class DitherAudio(nn.Module):
             form of noise used to randomize quantization error.
 
         Args:
-            dither : Amount of dither to add.
+            dither: Amount of dither to add.
         """
         super().__init__()
         self.dither = dither
@@ -103,11 +108,10 @@ class DitherAudio(nn.Module):
     def forward(self, x: torch.Tensor) -> torch.Tensor:
         """
         Args:
-            x : Tensor of shape (batch, time)
+            x: Tensor of shape (batch, time)
         """
         if self.training:
-            mask = x > 0.0
-            return x + mask * (self.dither * torch.randn_like(x))
+            return x + (self.dither * torch.randn_like(x))
         else:
             return x
 
@@ -122,7 +126,7 @@ class PreEmphasisFilter(nn.Module):
         `y[n] = y[n] - preemph * y[n-1]`
 
         Args:
-            preemph : Filter control factor.
+            preemph: Filter control factor.
         """
         super().__init__()
         self.preemph = preemph
@@ -131,7 +135,7 @@ class PreEmphasisFilter(nn.Module):
     def forward(self, x: torch.Tensor) -> torch.Tensor:
         """
         Args:
-            x : Tensor of shape (batch, time)
+            x: Tensor of shape (batch, time)
         """
         return torch.cat(
             (x[:, 0].unsqueeze(1), x[:, 1:] - self.preemph * x[:, :-1]), dim=1
@@ -149,9 +153,9 @@ class PowerSpectrum(nn.Module):
         method as used in NEMO.
 
         Args:
-            n_window_size : Number of elements in the window size.
-            n_window_stride : Number of elements in the window stride.
-            n_fft : Number of fourier features.
+            n_window_size: Number of elements in the window size.
+            n_window_stride: Number of elements in the window stride.
+            n_fft: Number of fourier features.
 
         Raises:
             ValueError: Raised when incompatible parameters are passed.
@@ -173,11 +177,17 @@ class PowerSpectrum(nn.Module):
         # doesnt support fft, like mobile or onnx.
         self.stft_func = torch.stft
 
+    def get_sequence_length(self, lengths: torch.Tensor) -> torch.Tensor:
+        seq_len = torch.floor(lengths / self.hop_length) + 1
+        return seq_len.to(dtype=torch.long)
+
     @torch.no_grad()
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
+    def forward(
+        self, x: torch.Tensor, lengths: torch.Tensor
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
         """
         Args:
-            x : Tensor of shape (batch, time)
+            x: Tensor of shape (batch, time)
         """
         x = self.stft_func(
             x,
@@ -193,7 +203,7 @@ class PowerSpectrum(nn.Module):
         x = torch.sqrt(x.pow(2).sum(-1))
         # get power spectrum
         x = x.pow(2.0)
-        return x
+        return x, self.get_sequence_length(lengths)
 
 
 class MelScale(nn.Module):
@@ -205,15 +215,15 @@ class MelScale(nn.Module):
         Also converts to log scale.
 
         Args:
-            sample_rate : Sampling rate of the signal
-            n_fft : Number of fourier features
-            nfilt : Number of output mel filters to use
-            log_scale : Controls if the output should also be applied a log scale.
+            sample_rate: Sampling rate of the signal
+            n_fft: Number of fourier features
+            nfilt: Number of output mel filters to use
+            log_scale: Controls if the output should also be applied a log scale.
         """
         super().__init__()
 
         filterbanks = (
-            create_fb_matrix(
+            melscale_fbanks(
                 int(1 + n_fft // 2),
                 n_mels=nfilt,
                 sample_rate=sample_rate,
@@ -232,59 +242,47 @@ class MelScale(nn.Module):
     def forward(self, x: torch.Tensor) -> torch.Tensor:
         """
         Args:
-            x : Tensor of shape (batch, features, time)
+            x: Tensor of shape (batch, features, time)
         """
         # dot with filterbank energies
         x = torch.matmul(self.fb.to(x.dtype), x)
         # log features
         # We want to avoid taking the log of zero
         if self.log_scale:
-            mask = x.abs() > 0.0
             x = torch.log(x + 2 ** -24)
-            x[~mask] = 0.0
         return x
 
 
-@dataclass
-class FilterbankConfig:
-    """Configuration to create [`FilterbankFeatures`][thunder.quartznet.transform.FilterbankFeatures]
-
-    Attributes:
-        sample_rate: Sampling rate of the signal. defaults to 16000.
-        n_window_size: Number of elements in the window size. defaults to 320.
-        n_window_stride: Number of elements in the window stride. defaults to 160.
-        n_fft: Number of fourier features. defaults to 512.
-        preemph: Preemphasis filtering control factor. defaults to 0.97.
-        nfilt: Number of output mel filters to use. defaults to 64.
-        dither: Amount of dither to add. defaults to 1e-5.
-    """
-
-    sample_rate: int = 16000
-    n_window_size: int = 320
-    n_window_stride: int = 160
-    n_fft: int = 512
-    preemph: float = 0.97
-    nfilt: int = 64
-    dither: float = 1e-5
-
-
-def FilterbankFeatures(cfg: FilterbankConfig) -> nn.Module:
+def FilterbankFeatures(
+    sample_rate: int = 16000,
+    n_window_size: int = 320,
+    n_window_stride: int = 160,
+    n_fft: int = 512,
+    preemph: float = 0.97,
+    nfilt: int = 64,
+    dither: float = 1e-5,
+) -> nn.Module:
     """Creates the Filterbank features used in the Quartznet model.
 
     Args:
-        cfg: required config to create instance
+        sample_rate: Sampling rate of the signal.
+        n_window_size: Number of elements in the window size.
+        n_window_stride: Number of elements in the window stride.
+        n_fft: Number of fourier features.
+        preemph: Preemphasis filtering control factor.
+        nfilt: Number of output mel filters to use.
+        dither: Amount of dither to add.
     Returns:
         Module that computes the features based on raw audio tensor.
     """
-    return nn.Sequential(
-        DitherAudio(dither=cfg.dither),
-        PreEmphasisFilter(preemph=cfg.preemph),
+    return MultiSequential(
+        Masked(DitherAudio(dither=dither), PreEmphasisFilter(preemph=preemph)),
         PowerSpectrum(
-            n_window_size=cfg.n_window_size,
-            n_window_stride=cfg.n_window_stride,
-            n_fft=cfg.n_fft,
+            n_window_size=n_window_size,
+            n_window_stride=n_window_stride,
+            n_fft=n_fft,
         ),
-        MelScale(sample_rate=cfg.sample_rate, n_fft=cfg.n_fft, nfilt=cfg.nfilt),
+        Masked(MelScale(sample_rate=sample_rate, n_fft=n_fft, nfilt=nfilt)),
         FeatureBatchNormalizer(),
     )
 
@@ -295,10 +293,10 @@ def patch_stft(filterbank: nn.Module) -> nn.Module:
     directly on arm cpu's, inside mobile applications.
 
     Args:
-        filterbank : the FilterbankFeatures layer to be patched
+        filterbank: the FilterbankFeatures layer to be patched
 
     Returns:
         Layer with the stft operation patched.
     """
-    filterbank[2].stft_func = convolution_stft
+    filterbank[1].stft_func = convolution_stft
     return filterbank
